"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_promise_1 = require("fs-promise");
function Endpoints() {
    let watcher;
    return {
        start({ config }, restart, stop) {
            return __awaiter(this, void 0, void 0, function* () {
                const endpointsFilePath = getEndpointsFilePath(config);
                yield fs_promise_1.ensureFile(endpointsFilePath);
                const endpoints = yield read(endpointsFilePath);
                watcher = fs_promise_1.watch(endpointsFilePath, restart);
                watcher.on('error', stop);
                return {
                    getServiceEndpoint: alias => getServiceEndpoint(endpoints, alias, config.normalize),
                    getServiceAddress: alias => getServiceAddress(endpoints, alias, config.normalize)
                };
            });
        },
        stop() {
            return __awaiter(this, void 0, void 0, function* () {
                if (watcher) {
                    watcher.close();
                    watcher = undefined;
                }
            });
        }
    };
}
exports.Endpoints = Endpoints;
exports.default = Endpoints;
function read(path) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield fs_promise_1.readJson(path);
        }
        catch (err) {
            return {};
        }
    });
}
function getEndpointsFilePath(config) {
    const { endpointsFilePath } = config;
    if (!endpointsFilePath)
        return './endpoints.json';
    return typeof endpointsFilePath === 'string' ? endpointsFilePath : endpointsFilePath.endpointsFilePath;
}
function getServiceEndpoint(endpoints, alias, normalize = false) {
    return normalize ? normalizeEndpoint(endpoints.currentHost, getByAlias(endpoints, alias)) : getByAlias(endpoints, alias);
}
function getServiceAddress(endpoints, alias, normalize = false) {
    return join(getServiceEndpoint(endpoints, alias, normalize));
}
function getByAlias(endpoints = {}, alias) {
    const hosts = endpoints.hosts || [];
    const host = hosts.find(host => host.alias === alias);
    return host && host.endpoint ? host.endpoint : resolveAddress(alias);
}
exports.getByAlias = getByAlias;
function resolveAddress(address) {
    const [host, port] = address.split(':').filter(_ => _);
    return { host, port };
}
function join(endpoint) {
    const { host, port } = endpoint;
    return [host, port].filter(_ => _).join(':');
}
function normalizeEndpoint(currentHost, endpoint) {
    return currentHost === endpoint.host ?
        { host: 'localhost', port: endpoint.port } : endpoint;
}
